<?php
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

class MY_Pagination extends CI_Pagination
{

  /**
   * With current uri and passed uri_segment, generate the correct navlink for a given offset 
   *
   * @access	public
   * @return	string
   */	
  function prep_navlink($offset)
  {
      $CI =& get_instance();
    
      if (($this->base_url == "") && ($this->uri_segment != ""))
      {
        $segment_array = $CI->uri->segment_array();
        $segment_array[$this->uri_segment] = (string)$offset;
        return site_url(join("/",$segment_array));
      } else {
        
        $this->base_url = preg_replace("/(.+?)\/*$/", "\\1/",  $this->base_url);
        if ($CI->config->item('url_suffix') != "") {
          $this->base_url = preg_replace("|" . preg_quote($CI->config->item('url_suffix')) . "/$|", "/",  $this->base_url);
        }
        return $this->base_url.$offset.$CI->config->item('url_suffix');
      
      }
  }


  /**
   * Generate the pagination links
   *
   * @access	public
   * @return	string
   */	
  function create_links()
  {
    // If our item count or per-page total is zero there is no need to continue.
    if ($this->total_rows == 0 OR $this->per_page == 0)
    {
       return '';
    }

    // Calculate the total number of pages
    $num_pages = ceil($this->total_rows / $this->per_page);


    // Determine the current page number.		
    $CI =& get_instance();	
    if ($CI->uri->segment($this->uri_segment) != 0)
    {
      $this->cur_page = $CI->uri->segment($this->uri_segment);
      
      // Prep the current page - no funny business!
      $this->cur_page = preg_replace("/[a-z\-]/", "", $this->cur_page);
    }
        
    if ( ! is_numeric($this->cur_page))
    {
      $this->cur_page = 0;
    }
    
    
    // Is there only one page? Hm... nothing more to do here then.
    if ($num_pages == 1 && $this->cur_page==0)
    {
      return '';
    }


    
    // Is the page number beyond the result range?
    // If so we show the last page
    if ($this->cur_page > $this->total_rows)
    {
      $this->cur_page = ($num_pages - 1) * $this->per_page;
    }
    
    $uri_page_number = $this->cur_page;
    $this->cur_page = floor(($this->cur_page/$this->per_page) + 1);

    // Calculate the start and end numbers. These determine
    // which number to start and end the digit links with
    $start = (($this->cur_page - $this->num_links) > 0) ? $this->cur_page - ($this->num_links - 1) : 1;
    $end   = (($this->cur_page + $this->num_links) < $num_pages) ? $this->cur_page + $this->num_links : $num_pages;

    // Add a trailing slash to the base URL if needed
    $this->base_url = preg_replace("/(.+?)\/*$/", "\\1/",  $this->base_url);
    
      // And here we go...
    $output = '';

    // Render the "First" link
    if($this->last_link)
    {    
        if  ($this->cur_page > $this->first_link)
        {
          $output .= $this->first_tag_open.'<a href="'.$this->prep_navlink(0).'">'.$this->first_link.'</a>'.$this->first_tag_close;
        }
    }
    // Render the "previous" link
    if  (($this->cur_page - $this->num_links) >= 0)
    {
      $i = $uri_page_number - $this->per_page;
      #if ($i == 0) $i = '';
      $output .= $this->prev_tag_open.'<a href="'.$this->prep_navlink($i).'">'.$this->prev_link.'</a>'.$this->prev_tag_close;
    }

    // Write the digit links
    for ($loop = $start -1; $loop <= $end; $loop++)
    {
      $i = ($loop * $this->per_page) - $this->per_page;
          
      if ($i >= 0)
      {
        if ($this->cur_page == $loop)
        {
          $output .= $this->cur_tag_open.$loop.$this->cur_tag_close; // Current page
        }
        else
        {
          $n = ($i == 0) ? '0' : $i;
          $output .= $this->num_tag_open.'<a href="'.$this->prep_navlink($n).'">'.$loop.'</a>'.$this->num_tag_close;
        }
      }
    }

    // Render the "next" link
    if ($this->cur_page < $num_pages)
    {
      $output .= $this->next_tag_open.'<a href="'.$this->prep_navlink($this->cur_page * $this->per_page).'">'.$this->next_link.'</a>'.$this->next_tag_close;
    }

    // Render the "Last" link
    if($this->last_link)
    {
        if (($this->cur_page + $this->num_links) < $num_pages)
        {
          $i = (($num_pages * $this->per_page) - $this->per_page);
          $output .= $this->last_tag_open.'<a href="'.$this->prep_navlink($i).'">'.$this->last_link.'</a>'.$this->last_tag_close;
        }
    }
    // Kill double slashes.  Note: Sometimes we can end up with a double slash
    // in the penultimate link so we'll kill all double slashes.
    $output = preg_replace("#([^:])//+#", "\\1/", $output);

    // Add the wrapper HTML if exists
    $output = $this->full_tag_open.$output.$this->full_tag_close;
    
    return $output;		
  }
}
?>